import { createGenerator } from '@unocss/core';
import markdownIt from 'markdown-it';
import { createHash } from 'node:crypto';
import { readFile, unlink, writeFile } from 'node:fs/promises';
import { basename, dirname, extname, resolve } from 'node:path';
import { fileURLToPath } from 'node:url';
import { renderToStaticMarkup } from 'react-dom/server';
import { minify } from 'terser';
import { body, header, page } from '../templates/page.js';
import { finalizeCss, transformCSSFile } from './css.js';
import { getTalk, getTalks, getTheme, rootDir } from './loader.js';
import { getCurrentMode } from './mode.js';
export const uncachedImportPrefix = '__uncached__';
export const markdownRenderer = markdownIt({
    html: true,
    breaks: true,
    linkify: true
});
// Bypass the Node.js ESM cache by temporarily copying the file to import in a file named after its hash in the same folder
async function uncachedImport(path) {
    const contents = await readFile(path);
    const md5 = createHash('sha1').update(contents).digest('hex');
    const tmpPath = resolve(dirname(path), `${uncachedImportPrefix}_${basename(path)}_${md5}${extname(path)}`);
    await writeFile(tmpPath, contents);
    try {
        const imported = await import(tmpPath);
        return imported;
    } finally{
        await unlink(tmpPath);
    }
}
export async function finalizeJs(code) {
    const { code: minified  } = await minify(code);
    return minified;
}
export function elapsedTime(startTime) {
    return (Number(process.hrtime.bigint() - startTime) / 1e6).toFixed(6);
}
export function parseContent(raw) {
    raw = raw === null || raw === void 0 ? void 0 : raw.toString();
    if (!raw) {
        return '';
    }
    return markdownRenderer.render(raw).replace(/^<p>/m, '').replace(/<\/p>$/m, '').trim();
}
export async function generateSlideset(theme, talk) {
    const { default: unoConfig  } = await uncachedImport(resolve(rootDir, 'src/themes', talk.config.theme, 'unocss.config.ts'));
    // Prepare the client
    const title = talk.document.title;
    const clientContext = {
        mode: getCurrentMode(),
        id: talk.id,
        title,
        dimensions: talk.config.dimensions,
        slidesCount: talk.slidesCount,
        slidesPadding: talk.slidesPadding,
        aspectRatio: talk.aspectRatio,
        current: 0
    };
    // Generate each slide
    const slides = [];
    for(let i = 0; i < talk.slides.length; i++){
        const slide = talk.slides[i];
        if (typeof slide.content === 'string') {
            slide.content = [
                slide.content
            ];
        }
        if (!slide.options) {
            slide.options = {};
        }
        if (!slide.classes) {
            slide.classes = {};
        }
        const { default: layout  } = await uncachedImport(resolve(rootDir, 'src/themes', talk.config.theme, 'layouts', (slide.layout ?? 'default') + '.tsx'));
        slides.push({
            ...layout({
                theme,
                talk,
                slide,
                index: i + 1
            }),
            key: `slide:${i + 1}`
        });
    }
    // Render the page body
    const contents = renderToStaticMarkup(body({
        slides
    }));
    // Extract the CSS
    const generator = createGenerator(unoConfig);
    const { css  } = await generator.generate(contents);
    // Generate theme style
    let themeCss = '';
    if (theme.style) {
        themeCss += await transformCSSFile(resolve(rootDir, 'src/themes', theme.id, theme.style), unoConfig);
    }
    // Generate the JS
    const client = await readFile(fileURLToPath(new URL('../assets/client.js', import.meta.url)), 'utf8');
    // Render the page
    const html = renderToStaticMarkup(page(title)).replace('@HEAD@', renderToStaticMarkup(header({
        talk,
        theme,
        css: await finalizeCss(unoConfig, themeCss + css, theme.fontsStyles),
        js: await finalizeJs(client.replace('const context = {}', `const context = ${JSON.stringify(clientContext)}`))
    }))).replace('@BODY@', contents);
    return html;
}
export async function generateSlidesets(context) {
    const slidesets = {};
    const talks = await getTalks();
    // For each talk, generate all the slideset
    for (const id of talks){
        const startTime = process.hrtime.bigint();
        const talk = await getTalk(id);
        const theme = await getTheme(talk.config.theme);
        slidesets[id] = await generateSlideset(theme, talk);
        context.log.info(`Generated slideset ${id} in ${elapsedTime(startTime)} ms`);
    }
    return slidesets;
}
